/*
 * @(#)jdemostate.java    1.00   02/21/02
 *
 * Copyright (c) 2002++ Agilis Software LLC, Santa Clara, CA, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Agilis Software LLC ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only
 * in accordance with the terms of the license agreement you entered
 * into with Agilis Software LLC.
 *
 * DISCLAIMER: this sample code is provided as-is.
 */

/*
 * This demo illustrates how to securely manage state in an
 * EasyLicenser-enabled application using the secure state management 
 * API's for maintaining quota consumption and other license-related 
 * application state.
 *
 * Running the demo once is equivalent to your end customer running your 
 * license-protected application at their site and consuming a specified 
 * amount of quota at their site.  The quota consumption is specified
 * as a command line argument.  Initialization of state accomplished
 * by running the demo with a quota consumption value of 0.
 * For illustration purposes, the demo saves state in a file in
 * the current directory.
 *
 * Features illustrated with this demo:
 * 1. How to use the full-featured "checkLicenseKeySecure" API call and 
 *    the key cookie mechanism to manage and update license state 
 *    including the built-in quota management and the extensibility 
 *    mechanism to manage application-specific state.
 * 2. How to utilize the application-password security mechanism.
 * 3. How to protect against system clock changes without writing code.
 *
 * For examples on using custom key handlers for extensibility,
 * as well as using digital signatures and other anti-spoofing measures,
 * see the accompanying "jdemosecure.java" demo application.
 *
 * Some important points to note about this demo:
 * 1. The code that is directly relevant to using the EasyLicenser API's
 *    is indicated as such in comments.  The rest of the code exists in
 *    order to have a functioning self-contained program.
 * 2. All code pertaining to reading the key from an EasyLicenser file is
 *    for functional purposes only.  It is not meant to suggest a specific
 *    technique for managing the key in your application.  Nor is it
 *    a part of the EasyLicenser product.  In particular, note that your
 *    actual application will typically have access to just the license key
 *    without any accompanying metadata about the key.
 * 3. The demo will work with keys generated with EasyLicenser 2.0 as well
 *    as prior versions.  The EasyLicenser 2.0 run time library is required.
 * 4. This demo performs automatic state management in a file that persists
 *    past its execution.  Therefore, each successive run will have a 
 *    different outcome and eventually you should encounter an "exceeded
 *    quota" error if you supplied a quota limited key.
 * 5. Unlike the basic "jdemo.java", this demo processes only the first key
 *    in its input file.  Therefore, provide a file containing only one key.
 *    The key need not be quota limited.  If it isn't, the key cookie will
 *    still manage a use count, which can be useful in an actual application.
 */


import java.io.*;
import java.text.*;
import java.util.*;
import com.vs.ezlicrun.*;

public class jdemostate {

	/* File types */
	static final int EZLM_FILE_TYPE_UNKNOWN	= 0;
	static final int EZLM_FILE_TYPE_TXT		= 1;
	static final int EZLM_FILE_TYPE_XML		= 2;
	static final int EZLM_FILE_TYPE_CSV		= 3;
	static final int EZLM_FILE_TYPE_TAB		= 4;
	static final int EZLM_FILE_TYPE_INI		= 5;
	static final int EZLM_FILE_TYPE_KEY		= 6;

	static final int KEY_MAXLENGTH = 4096;

	public jdemostate() {}

	/**
	 * main entry point:
	 * Accepts three mandatory command line arguments:
	 * 1. Name of a text file containing a license key to be validated.
	 * 2. Expected user / host name that should match what's in the key.
     *    If the key is known to have been generated with the "enforce OS
     *    user / host" flag, you can specify the special string "null" to
     *    signify a null value.
 	 * 3. Quota increment: amount by which to increment quota consumption.
 	 *    To reset the saved state, specify a value of 0 to simulate
 	 *    product install time.
	 * and two optional arguments (either none or both must be specified):
	 * 4. Case-sensitive product name that was specified at the time of key creation,
	 *    if the product had an associated application password
	 * 5. Product's associated password's case-sensitive public key, available with
	 *    the product definition and also obtainable by invoking PkCrypt.makePublicKey(password)
	 *    and with the "makepubkey" Java utility located at ezlic2.0/util.
	 */
	public static void main(String[] args) {
		if (args.length < 3 || args.length == 4) {
			System.out.println("*** Usage: jdemostate key-text-file-name user-host-name quota-incr [product-name application-password-public-key] ***");
			System.exit(1);
		}
		String keyfile = args[0];
		String userhost = args[1];
		long quotaUsageToDate = 0;
		try {
			quotaUsageToDate = Long.parseLong(args[2]);
			if (quotaUsageToDate < 0)
				throw new Exception();
		} catch (Throwable t) {
			System.out.println("*** Invalid quota increment '" + args[2] + "' - must be an integer >= 0 ***");
			System.out.println("*** Usage: jdemostate key-text-file-name user-host-name quota-incr [product-name application-password-public-key] ***");
			System.exit(1);
		}
		String productName = null;
		String appPwd = null;
		if (args.length > 3) {
			productName = args[3];
			appPwd = args[4];
		}
		System.out.println("\nLicense State Demo.  Running in " +
						   (quotaUsageToDate == 0 ? 
						   "application initialization":
						   "application execution") +
						   " mode.\n");
		System.out.println("Arguments:");
		System.out.println("  License key file: " + keyfile);
		System.out.println("  User / Host name: " + userhost);
		System.out.println("  Quota increment: " + quotaUsageToDate);
		System.out.println("  Product name:     " + (productName == null ? "<not specified>":productName));
		System.out.println("  App Passwd Public Key:     " + (appPwd == null ? "<not specified>":appPwd));

		String keyCookie = null;
		String appState = (new SimpleDateFormat("yyyy-MM-dd hh:mm:ss z")).format(new Date());

		if (quotaUsageToDate==0) {
			/* Simulate product and key installation: initialize key cookie and exit. */

			/* 
			 * Set app state to current date string, for illustration purposes.  In a real
			 * application, you can use the application state to store arbitrary licensing
			 * related information.
			 */
			try {
				/* EasyLicenser API call: create an initial key cookie at product install time */
				keyCookie = EzLicenseInfo.createKeyCookieSeed(productName, appPwd, appState, false);
			} catch (EzLicExceptionBase ex) {
				System.out.println("*** Error creating key cookie seed.  Exception: " + ex.getClass().getName() +
								   ", message: " + ex.getMessage());
				System.exit(2);
			}
			updateKeyCookieInFile(keyCookie);
			System.out.println("Initialized license state.  Exiting.");
			System.exit(0);
		}

		/* Application run time simulation: get the last key cookie from persistent store */
		keyCookie = getKeyCookieFromFile();
		System.out.println("Dump of key cookie before license key check:");
		displayKeyCookie(keyCookie, productName, appPwd);


		if (userhost.equals("null"))
			userhost = null;
		int fileType = getFileType(keyfile);
		Object[] ret = readFile(keyfile, 1000);
		if (ret == null) {
			System.out.println("*** Cannot process license key file due to errors.  Aborting. ***");
			System.exit(1);
		}
		int numlines = ((Integer)ret[1]).intValue();
		if (numlines == 0) {
			System.out.println("License key file is empty.  Nothing to do.  Exiting.");
			System.exit(0);
		}

		System.out.println("Checking first license key in file...");
		String[] filelines = (String[])ret[0];
		Object[] ret2 = null;
		if ((ret2 = getKey(filelines, numlines, 0, fileType)) == null)
			System.exit(1);

		String keytext = (String)ret2[0];
		System.out.println("License key:\n["+keytext+"]\n");

		// EasyLicenser API call (required): allocate a license context
		EzLicenseInfo licenseInfo = new EzLicenseInfo();
		int warnings = 0;
		try {
			// EasyLicenser API call (required): perform the license key check
			licenseInfo.checkLicenseKeySecure(
						keytext,		// the license key string
						keyCookie,		// previous key cookie
						(new MyExceptionHandler2()),	// custom exception handler
						null, null,		// no custom key handler 
										// for this example
						7,				// threshold days to trigger 
										// "about to expire" warning
						-quotaUsageToDate, 	// quota increment - note it
										// has to be specified as a -ve
										// incremental # in order for 
										// the automatic state management
										// to kick in.  Note also that
										// a -ve value implies an increment
						10,				// threshold balance quota to
										// trigger "about to exhaust
										// quota" warning
						365, 			// allow license key check to
										// succeed with warning instead
										// of error within these many
										// days past expiration date.
						100,			// allow license key check to
										// succeed with warning instead
										// of error within these many
										// uses past quota limit.
						userhost, 		// user / host name - can be null
										// if OS enforcement is in effect.
						0,				// current use level (note: different
										// from quota which is cumulative
										// and can use different units) -
										// checked against limit in key.
										// applies only to non-single-user
										// licenses.
						productName,	// name of product for which key
										// was generated
						appPwd,			// application password public key,
										// if the product had an application
										// password specified when the key was
										// generated.
						appState);		// new application state to save in
										// key cookie.
		} catch (EzLicExceptionBase ex) {
			if (ex instanceof MyExceptionHandler2) {
				// in practice this would be caught somewhere in the code
				// quite removed from the API call.
				keyCookie = ex.getMessage();	// extract new key cookie
				warnings = ex.getMessageCode(); // extract warning codes
				System.out.println("License key validation successful" + 
								   (warnings == 0?"":" with warnings") +
								   ".\n");
				displayLicenseParams(licenseInfo);
				checkWarnings(warnings);
				try {
					System.out.println("Dump of new key cookie after license key check:");
					displayKeyCookie(keyCookie, productName, appPwd);
					updateKeyCookieInFile(keyCookie);
				} catch (Throwable t) {
					System.out.println("*** License key post-processing failed ***\nError: " +
						t.getClass().getName() + ", Message: " + t.getMessage());
					System.exit(3);
				}
			} else {
				System.out.println("*** License key check failed ***\nError: " +
					ex.getClass().getName() + ", Message: " + ex.getMessage());
				System.exit(3);
			}
		}

		System.out.println("End of demo.");
		System.exit(0);
	}

	/*
	 * Display state contents in the encrypted key cookie.  Note that 
	 * you can introspect a key cookie but it can't be created / 
	 * updated outside the EasyLicenser API's.
	 * In an actual application, you may be interested in introspecting
	 * an application-specific subset of the cookie information, none
	 * in the simple case.
	 */
	static void displayKeyCookie(String keyCookie, 
								 String productName, 
								 String appPassword)
	{
		System.out.println("  Key cookie:\n[" + keyCookie + "]");
		try {
			/* EasyLicenser API call: get application state from key cookie */
			String appState = 
				EzLicenseInfo.
					getKeyCookieAppState(keyCookie, productName, appPassword);
			System.out.println("    Application state: [" + appState + "]");

			/* EasyLicenser API call: get quota-consumed-to-date state from key cookie */
			long quotaToDate = 
				EzLicenseInfo.
					getKeyCookieQuotaToDate(keyCookie, productName, appPassword);
			System.out.println("    Quota-to-date: " + quotaToDate);

			/* EasyLicenser API call: get cookie update timestamp from key cookie */
			long updateTs = 
				EzLicenseInfo.
					getKeyCookieTs(keyCookie, productName, appPassword);
			System.out.println("    Update date/time: " +
						(new SimpleDateFormat("yyyy-MM-dd hh:mm:ss z")).
						format(new Date(updateTs)) + " (timestamp: " +
						updateTs + ")");
		} catch (EzLicExceptionBase ex) {
			System.out.println("*** Error introspecting key cookie.  Exception: " +
				ex.getClass().getName() + ", Message: " +
				ex.getMessage());
			System.exit(2);
		}
	}


	/*
	 * Update key cookie in file.  For demo purposes only.  In an actual
	 * application, you'd persist your key cookie in an application-
	 * specific manner.
	 */
	static void updateKeyCookieInFile(String keyCookie)
	{
		try {
			File f = new File("mycookiefile");
			FileOutputStream fout = new FileOutputStream(f);
			PrintStream out = new PrintStream(fout);
			out.println(keyCookie);
			out.flush();
			out.close();
		} catch (Exception ex) {
			System.out.println("*** Error writing to key cookie file.  Exception: " +
				ex.getClass().getName() + ", Message: " +
				ex.getMessage());
			System.exit(2);
		}
	}

	/*
	 * Get key cookie from file.  For demo purposes only.  In an actual
	 * application, you'd persist your key cookie in an application-
	 * specific manner.
	 */
	static String getKeyCookieFromFile()
	{
		File cf = new File("mycookiefile");
		if (!cf.exists()) {
			System.out.println("*** File for key cookie does not exist ***");
			System.exit(2);
		}
		BufferedReader in = null;
		try {
			in = new BufferedReader(
					new InputStreamReader(
						new FileInputStream(cf)));
			String kookie = null;
			kookie = in.readLine();
			if (kookie == null) {
				System.out.println("*** Invalid key cookie file - empty ***");
				System.exit(3);
			}
			kookie = kookie.trim();
			if (kookie.length() > KEY_MAXLENGTH) {
				System.out.println("*** Invalid key cookie: string too long. ***");
				System.exit(1);
			}
			return kookie;
		} catch (Exception ex) {
			System.out.println("*** IO Error reading key cookie file ***");
			System.out.println("Exception: " + ex.getClass().getName() + ", Message: " + ex.getMessage());
			ex.printStackTrace();
			System.exit(2);
		} finally {
			try {
				in.close();
			} catch (Exception exx) {
			}
		}
		return null;
	}

	static void checkWarnings(int warnings)
	{
		if (warnings != 0) {
			System.out.println("    Warnings:");
			if ((warnings & EzLicExceptionBase.EZLIC_WARN_EXPIRING) != 0)
				System.out.println("        Time Limited Key Is About To Expire");
			if ((warnings & EzLicExceptionBase.EZLIC_WARN_QUOTA) != 0)
				System.out.println("        Quota Limited Key Is About To Expire");
			if ((warnings & EzLicExceptionBase.EZLIC_WARN_EXPIRED) != 0)
				System.out.println("        Time Limited Key Has Expired");
			if ((warnings & EzLicExceptionBase.EZLIC_WARN_QUOTA_EXCEEDED) != 0)
				System.out.println("        Quota Limited Key Has Expired");
			if ((warnings & EzLicExceptionBase.EZLIC_WARN_EXP_GRACE_TRUNC) != 0)
				System.out.println("        Expiration Days Grace In API Call Too Large - Truncated To 5 / 25% Of Lifespan");
			if ((warnings & EzLicExceptionBase.EZLIC_WARN_QUOTA_GRACE_TRUNC) != 0)
				System.out.println("        Quota Grace In API Call Too Large - Truncated To 5 / 25% Of Original Quota");
			System.out.println("");
		}
	}

	static void displayLicenseParams(EzLicenseInfo licenseInfo)
	{
		System.out.println("    License Key Information:");
		System.out.println("        Key Version: " + 
							(licenseInfo.getKeyVersion() <= 0 ? "Legacy (1.1 or earlier)" :
							 String.valueOf(licenseInfo.getKeyVersion()/10)+"." + 
							 String.valueOf(licenseInfo.getKeyVersion()%10)
							));
		if (licenseInfo.getChainedKeyId() != null)
			System.out.println("Replacement For Key With Id: " + licenseInfo.getChainedKeyId());
		showLicenseModel(licenseInfo);
		int bitmap = licenseInfo.getLicenseModeBitmap();
		System.out.println("        License Mode Bitmap: " + bitmap);
		if ((bitmap & EzLicenseInfo.EZLIC_MODE_TIME) != 0)
			System.out.println("            TIME LIMITED");
		if ((bitmap & EzLicenseInfo.EZLIC_MODE_OPTIONS) != 0)
			System.out.println("            OPTIONS");
		if ((bitmap & EzLicenseInfo.EZLIC_MODE_METERED) != 0)
			System.out.println("            METERED");
		if ((bitmap & EzLicenseInfo.EZLIC_MODE_CUSTOM_KEY) != 0)
			System.out.println("            CUSTOM KEY");
		if ((bitmap & EzLicenseInfo.EZLIC_MODE_CUSTOM_COOKIE) != 0)
			System.out.println("            CUSTOM COOKIE");
		long lvalue;
		lvalue = licenseInfo.getUsageValue();
		if (lvalue > 0)
			System.out.println("        Usage Limit Value: " + lvalue);
		lvalue = licenseInfo.getQuotaValue();
		if (lvalue > 0)
			System.out.println("        Quota Value: " + lvalue);
		String options = licenseInfo.getOptions();
		if (options != null && !options.equals(""))
			System.out.println("        Options: [" + options + "]");
		Date expDate = licenseInfo.getExpirationDate();
		if (expDate != null)
			System.out.println("        Expiration Date: " +
							(new SimpleDateFormat("yyyy-MM-dd hh:mm:ss z")).
							format(expDate));
		System.out.println("        User / host name: " +
						licenseInfo.getUserHostNetName());
		if (licenseInfo.getEnforce())
			System.out.println("        Operating system check enforced.");
		String customCookie = licenseInfo.getCustomCookie();
		if (customCookie != null && !customCookie.equals(""))
			System.out.println("        Custom Cookie: [" + customCookie + "]");
		String customKey = licenseInfo.getCustomKey();
		if (customKey != null && !customKey.equals(""))
			System.out.println("        Custom Key: [" + customKey + "]");
		if (licenseInfo.getAppPwd() != null && !licenseInfo.getAppPwd().equals(""))
			System.out.println("        Application Password: [" + licenseInfo.getAppPwd() + "]");
	}

	static void showLicenseModel(EzLicenseInfo licenseInfo)
	{
		int licenseModel = licenseInfo.getLicenseModelCode();
		int licenseType = licenseInfo.getLicenseTypeCode();
		System.out.println("        License Model: " +
						   (licenseModel == EzLicenseInfo.EZLIC_MODEL_USER?"User":"Server"));
		System.out.print("        License Type: ");
		switch(licenseModel) {
		case EzLicenseInfo.EZLIC_MODEL_USER:
			switch(licenseType) {
			case EzLicenseInfo.EZLIC_TYPE_U_USER:
				System.out.println("Named User");
				break;
			case EzLicenseInfo.EZLIC_TYPE_U_NODE:
				System.out.println("Node Locked");
				break;
			case EzLicenseInfo.EZLIC_TYPE_U_FLOAT:
				System.out.println("Floating User");
				break;
			default:
				System.out.println("*** Unknown license type " + licenseType + " ***");
			}
			break;
		case EzLicenseInfo.EZLIC_MODEL_SERVER:
			switch(licenseType) {
			case EzLicenseInfo.EZLIC_TYPE_SVR_CONC:
				System.out.println("Concurrent User");
				break;
			case EzLicenseInfo.EZLIC_TYPE_SVR_CPU:
				System.out.println("CPU Count");
				break;
			case EzLicenseInfo.EZLIC_TYPE_SVR_MHZ:
				System.out.println("CPU MHZ");
				break;
			case EzLicenseInfo.EZLIC_TYPE_SVR_NMU:
				System.out.println("Named User");
				break;
			default:
				System.out.println("*** Unknown license type " + licenseType + " ***");
			}
			break;
		default:
			System.out.println("*** Unknown license model " + licenseModel + " ***");
		}
	}

	/* 
	 * The file IO is for convenience purposes only - you'd manage the persistence of 
	 * your key according to the requirements of your application.
	 */

	static Object[] readFile(String filename, int maxfilelines)
	{
		Object[] ret = new Object[2];
		String[] filelines = new String[maxfilelines];
		ret[0] = filelines;
		ret[1] = null;
		File kf = new File(filename);
		if (!kf.exists()) {
			System.out.println("*** License key file '" +
						filename + "' not found ***");
			System.exit(2);
		}
		int numlines = 0;
		BufferedReader in = null;
		try {
			in = new BufferedReader(
					new InputStreamReader(
						new FileInputStream(kf)));
			String keyline = null;
			while ((keyline = in.readLine()) != null && numlines < maxfilelines)
				filelines[numlines++] = keyline.trim();
			if (keyline != null) {
				System.out.println("*** Invalid key: too many lines.  Limit is " + maxfilelines + ". ***");
				System.exit(1);
			}
			filelines[numlines] = null;
		} catch (Exception ex) {
			System.out.println("*** IO Error reading license key file ***");
			System.out.println("Exception: " + ex.getClass().getName() + ", Message: " + ex.getMessage());
			//ex.printStackTrace();
			System.exit(2);
		} finally {
			try {
				in.close();
			} catch (Exception exx) {
			}
		}
		ret[1] = new Integer(numlines);
		return ret;
	}


	static Object[] getKey(String filelines[], int numlines,
						   int currLine, int fileType)
	{
		Object[] ret = new Object[2];
		ret[0] = null;
		ret[1] = null;
		int idx = -1;
		String keytext = null;

		if (currLine >= numlines)
			return null;

		if (currLine == 0) {
			/* check / adjust for first line */
			switch (fileType) {
			case EZLM_FILE_TYPE_TXT:
				if (!(filelines[currLine++].startsWith("License Key List:") &&
				      filelines[currLine++].startsWith("----------------"))) {
					System.out.println("*** Not a valid EasyLicenser text export file ***");
					System.out.println("Bad line #: " + currLine + ", contents: " + filelines[currLine-1]);
					System.exit(2);
				}
				currLine += 3; /* position at "Key Id:" */
				break;
			case EZLM_FILE_TYPE_XML:
				if (!(filelines[currLine++].startsWith("<?xml") &&
				      filelines[currLine++].startsWith("<key-list>"))) {
					System.out.println("*** Not a valid EasyLicenser XML export file ***");
					System.out.println("Bad line #: " + currLine + ", contents: " + filelines[currLine-1]);
					System.exit(2);
				}
			default:
				break;
			}
		} else {
			/* check for EOF */
			switch (fileType) {
			case EZLM_FILE_TYPE_XML:
				if (filelines[currLine] != null && 
				    filelines[currLine].startsWith("</key-list>"))
					return null;
			default:
				if (filelines[currLine] == null)
					return null;
				break;
			}
		}
		/* get key */
		switch (fileType) {
		case EZLM_FILE_TYPE_TXT:
			for (; !filelines[currLine].startsWith("License key:") && 
			       currLine < numlines; 
			     currLine++);
			if (++currLine >= numlines) {
				System.out.println("*** Premature termination of text file ***");
				return null;
			}
			keytext = filelines[currLine].trim();
			currLine += 2;
			break;
		case EZLM_FILE_TYPE_XML:
			if (!filelines[currLine].startsWith("<key id=")) {
				System.out.println("*** Invalid XML record - treating as EOF ***");
				System.out.println("Bad line #: " + currLine+1 + ", contents: " + filelines[currLine]);
				return null;
			}
			if (++currLine >= numlines) {
				System.out.println("*** Premature termination of XML file ***");
				return null;
			}
			keytext = filelines[currLine].trim();
			currLine++;
			currLine++;
			break;
		case EZLM_FILE_TYPE_INI:
			if ((idx = filelines[currLine].indexOf("=")) <= 0) {
				System.out.println("*** Invalid property file format - treating as EOF ***");
				return null;
			}
			keytext = filelines[currLine].substring(idx+1).trim();
			currLine++;
			break;
		case EZLM_FILE_TYPE_KEY:
			keytext = filelines[currLine].trim();
			currLine++;
			break;
		case EZLM_FILE_TYPE_CSV:
		case EZLM_FILE_TYPE_TAB:
			idx = filelines[currLine].lastIndexOf(
						(fileType==EZLM_FILE_TYPE_CSV?",":"\t"));
			if (idx <= 0) {
				System.out.println("*** Invalid " +
						(fileType==EZLM_FILE_TYPE_CSV?"csv":"tab") +
							" file format - treating as EOF ***");
				System.out.println("Bad line #: " + currLine+1 + ", contents: " + filelines[currLine]);
				return null;
			}
			keytext = filelines[currLine].substring(idx+1).trim();
			currLine++;
			break;
		}
		if (keytext.length() > KEY_MAXLENGTH) {
			System.out.println("*** Key length exceeds limit of " + KEY_MAXLENGTH + " characters ***");
			System.exit(2);
		}
		ret[0] = strip(keytext);
		ret[1] = new Integer(currLine);
		return ret;
	}

	static int getFileType(String filename) {
		int dotpos = filename.lastIndexOf('.');
		if (dotpos <= 0 || dotpos > filename.length()-4) {
			return EZLM_FILE_TYPE_UNKNOWN;
		}
		String suffix = filename.substring(dotpos+1);
		if (suffix.equals("txt"))
			return EZLM_FILE_TYPE_TXT;
		if (suffix.equals("xml"))
			return EZLM_FILE_TYPE_XML;
		if (suffix.equals("csv"))
			return EZLM_FILE_TYPE_CSV;
		if (suffix.equals("tab"))
			return EZLM_FILE_TYPE_TAB;
		if (suffix.equals("ini"))
			return EZLM_FILE_TYPE_INI;
		if (suffix.equals("key"))
			return EZLM_FILE_TYPE_KEY;
		System.out.println("*** Unknown file type: '" + suffix + "'.  Exiting ***");
		System.exit(1);
		return -1; // braindead compiler
	}

	static String strip(String instr) {
		String retstr = "";
		try {
			StringTokenizer stk = new StringTokenizer(instr," \n\t");
			while (stk.hasMoreTokens()) {
				String tokstr = stk.nextToken();
				retstr += tokstr;
			}
		} catch (Exception ex) {
			System.out.println("*** Got exception: " + ex.getMessage());
		}
		return retstr;
	}
	
}

// Custom exception handler for signalling success
class MyExceptionHandler2 extends EzLicExceptionBase {
	MyExceptionHandler2() {}
}
